//-------------------------------------------------------------------------------
// CCamara.cpp
//-------------------------------------------------------------------------------


#include "CCamara.h"

//------------------------------------------------------------------------------
// Funcin: Constructor
// Propsito: Inicia los valores por defecto de la clase.
//------------------------------------------------------------------------------
CCamara::CCamara()
{
	m_fFov = D3DXToRadian(45.0f);

	m_iAncho = WND_WIDTH;
	m_iAlto = WND_HEIGHT;

	m_fPitch = m_fYaw = m_fRoll = 0.0f;

	m_fPlanoCercano = 1.0f;
	m_fPlanoLejano = 9999.9f;

	m_Posicion = D3DXVECTOR3(0.0f, 0.0f, -100.0f);

	m_Look = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	m_Dcha = D3DXVECTOR3(1.0f, 0.0f, 0.0f);
	m_Arriba = D3DXVECTOR3(0.0f, 1.0f, 1.0f);
}

//------------------------------------------------------------------------------
// Funcin: Destructor
// Propsito: Elimina todo lo necesario.
//------------------------------------------------------------------------------
CCamara::~CCamara()
{
}

//------------------------------------------------------------------------------
// Funcin: ActualizaVista
// Propsito: Crea una matriz de vista adecuada a los valores especificados 
// y la activa.
//------------------------------------------------------------------------------
void CCamara::ActualizarVista(LPDIRECT3DDEVICE9 pD3DDev)
{
	D3DXMATRIX T, Rx, Ry, Rz;
	D3DXMATRIX view;

	// Cogemos el vector dcha, arriba y lookat actualizados
	pD3DDev->GetTransform(D3DTS_VIEW, &view);
	m_Dcha = D3DXVECTOR3(view._11, view._21, view._31);
	m_Arriba = D3DXVECTOR3(view._12, view._22, view._32);
	m_Look = D3DXVECTOR3(view._13, view._23, view._33);

	// Calculamos la nueva vista
	D3DXMatrixTranslation(&T, -m_Posicion.x, -m_Posicion.y, -m_Posicion.z);
	D3DXMatrixRotationY(&Ry, -m_fYaw);
	D3DXMatrixRotationX(&Rx, -m_fPitch);
	D3DXMatrixRotationZ(&Rz, -m_fRoll);

	view = T * Ry * Rx * Rz;

	// La activamos
	pD3DDev->SetTransform(D3DTS_VIEW, &view);
}

//------------------------------------------------------------------------------
// Funcin: ActualizaProyeccion
// Propsito: Crea una matriz de proyeccin en perspectiva adecuada a los valores 
// especificados y la activa.
//------------------------------------------------------------------------------
void CCamara::ActualizarProyeccion(LPDIRECT3DDEVICE9 pD3DDev)
{
	D3DXMATRIX matProj;
	D3DXMatrixPerspectiveFovLH(&matProj, m_fFov, m_iAncho/(float)m_iAlto, m_fPlanoCercano, m_fPlanoLejano);
	pD3DDev->SetTransform(D3DTS_PROJECTION, &matProj);
}

//------------------------------------------------------------------------------
// Funcin: PonerFov
// Propsito: Asigna el FOV (field of view) de la cmara (para la matriz de proyeccin).
//------------------------------------------------------------------------------
void CCamara::PonerFov(float fFov)
{
	m_fFov = fFov;
}

//------------------------------------------------------------------------------
// Funcin: DaFov
// Propsito: Devuelve el FOV (field of view) de la cmara(para la matriz de proyeccin).
//------------------------------------------------------------------------------
float CCamara::DaFov(void)
{
	return m_fFov;
}

//------------------------------------------------------------------------------
// Funcin: PonerDimensiones
// Propsito: Asigna las dimensiones de la pantalla.
//------------------------------------------------------------------------------
void CCamara::PonerDimensiones(int iWidth, int iAlto)
{
	m_iAncho = iWidth;
	m_iAlto= iAlto;
}

//------------------------------------------------------------------------------
// Funcin: PonerPlanos
// Propsito: Asigna los planos cercano y lejano del view frustum (para la 
// matriz de proyeccin).
//------------------------------------------------------------------------------
void CCamara::PonerPlanos(float fNear, float fFar)
{
	m_fPlanoCercano = fNear;
	m_fPlanoLejano = fFar;
}

//------------------------------------------------------------------------------
// Funcin: Avanza
// Propsito: Avanza (fSpeed > 0) / Retrocede (fSeed < 0) la cmara.
//------------------------------------------------------------------------------
void CCamara::Avanzar(float fSpeed, float fDTime)
{
	m_Posicion += fSpeed * fDTime * m_Look;
}

//------------------------------------------------------------------------------
// Funcin: Desplazamiento
// Propsito: Desplazamiento a Derecha (fSpeed > 0) / Izquierda (fSeed < 0) la cmara.
//------------------------------------------------------------------------------
void CCamara::Desplazar(float fSpeed, float fDTime)
{
	m_Posicion += fSpeed * fDTime * m_Dcha;
}

//------------------------------------------------------------------------------
// Funcin: Sube
// Propsito: Elevacin (fSpeed > 0) / Descenso (fSeed < 0) de la cmara.
//------------------------------------------------------------------------------
void CCamara::Subir(float fSpeed, float fDTime)
{
	m_Posicion += fSpeed * fDTime * m_Arriba;
}

//------------------------------------------------------------------------------
// Funcin: PonerPosAbs
// Propsito: Especifica la posicin absoluta de la cmara.
//------------------------------------------------------------------------------
void CCamara::PonerPosAbs(D3DXVECTOR3 &pos)
{
	m_Posicion = pos;
}

//------------------------------------------------------------------------------
// Funcin: DaPosicion
// Propsito: Devuelve la posicin de la cmara.
//------------------------------------------------------------------------------
D3DXVECTOR3 CCamara::DaPosicion(void)
{
	return m_Posicion;
}

//------------------------------------------------------------------------------
// Funcin: PonerOrientacion
// Propsito: Especifica los ngulos (Pitch, Yaw, Roll) de la cmara.
//------------------------------------------------------------------------------
void CCamara::PonerOrientacion(float fPitch, float fYaw, float fRoll)
{
	m_fPitch = fPitch;
	m_fYaw = fYaw;
	m_fRoll = fRoll;
}

//------------------------------------------------------------------------------
// Funcin: DaOrientacion
// Propsito: Devuelve los ngulos (Pitch, Yaw, Roll) de la cmara.
//------------------------------------------------------------------------------
D3DXVECTOR3 CCamara::DaOrientacion(void)
{
	return D3DXVECTOR3(m_fPitch, m_fYaw, m_fRoll);
}

//------------------------------------------------------------------------------
// Funcin: DaVectorDcha
// Propsito: Devuelve el vector "Dcha" de la cmara.
//------------------------------------------------------------------------------
D3DXVECTOR3	CCamara::DaVectorDcha(void)
{
	return m_Dcha;
}

//------------------------------------------------------------------------------
// Funcin: DaVectorArriba
// Propsito: Devuelve el vector "Arriba" de la cmara.
//------------------------------------------------------------------------------
D3DXVECTOR3	CCamara::DaVectorArriba(void)
{
	return m_Arriba;
}

//------------------------------------------------------------------------------
// Funcin: DaVectorLookAt
// Propsito: Devuelve el vector "LookAt" de la cmara.
//------------------------------------------------------------------------------
D3DXVECTOR3	CCamara::DaVectorLookAt(void)
{
	return m_Look;
}